@@ -26,10 +26,14 @@ class MyRoot(controllers.RootController):
         self.errors = tg_errors
         return "Error Message"
 
-    [expose(html="turbogears.tests.simple", allow_json=True)]
+    [expose(template="turbogears.tests.simple", allow_json=True)]
     def test(self):
         return dict(title="Foobar", mybool=False, someval="niggles")
 
+    [expose(html="turbogears.tests.simple")]
+    def test_deprecated(self):
+        return dict(title="Oldbar", mybool=False, someval="giggles")
+
     [expose()]
     def invalid(self):
         return None
@@ -62,7 +66,7 @@ class MyRoot(controllers.RootController):
         return dict(title="Foobar", mybool=False, someval="foo",
             tg_template="turbogears.tests.simple")
 
-    [expose(html="turbogears.tests.simple")]
+    [expose(template="turbogears.tests.simple")]
     def exposetemplate_short(self):
         return dict(title="Foobar", mybool=False, someval="foo")
 
@@ -84,15 +88,15 @@ class MyRoot(controllers.RootController):
         return "No Error"
     errorchain = error_handler(istrue)(errorchain)
 
-    [expose(format="json", html="turbogears.tests.simple")]
+    [expose(format="json", template="turbogears.tests.simple")]
     def returnjson(self):
         return dict(title="Foobar", mybool=False, someval="foo",
-            tg_html="turbogears.tests.simple")
+            tg_template="turbogears.tests.simple")
 
-    [expose(html="turbogears.tests.simple", allow_json=False)]
+    [expose(template="turbogears.tests.simple", allow_json=False)]
     def allowjson(self):
         return dict(title="Foobar", mybool=False, someval="foo",
-             tg_html="turbogears.tests.simple")
+             tg_template="turbogears.tests.simple")
 
     [expose(format="json")]
     def impliedjson(self):
@@ -102,7 +106,7 @@ class MyRoot(controllers.RootController):
     def explicitjson(self):
         return dict(title="Blub")
 
-    [expose(html="turbogears.tests.simple", allow_json=True)]
+    [expose(template="turbogears.tests.simple", allow_json=True)]
     def jsonerror_handler(self):
         return dict(someval="errors")
 
@@ -138,11 +142,11 @@ class MyRoot(controllers.RootController):
         return self.save(submit, firstname, lastname)
     save2 = error_handler(validation_error_handler)(save2)
 
-    [expose(html="turbogears.tests.simple")]
+    [expose(template="turbogears.tests.simple")]
     def useother(self):
         return dict(tg_template="turbogears.tests.othertemplate")
 
-    [expose(html="cheetah:turbogears.tests.simplecheetah")]
+    [expose(template="cheetah:turbogears.tests.simplecheetah")]
     def usecheetah(self):
         return dict(someval="chimps")
 
@@ -151,32 +155,32 @@ class MyRoot(controllers.RootController):
         self.rwt_called += 1
         func(*args, **kw)
 
-    [expose(html="turbogears.tests.simple", allow_json=True)]
+    [expose(template="turbogears.tests.simple", allow_json=True)]
     def flash_plain(self):
         flash("plain")
         return dict(title="Foobar", mybool=False, someval="niggles")
 
-    [expose(html="turbogears.tests.simple", allow_json=True)]
+    [expose(template="turbogears.tests.simple", allow_json=True)]
     def flash_unicode(self):
         flash(u"\xfcnicode")
         return dict(title="Foobar", mybool=False, someval="niggles")
 
-    [expose(html="turbogears.tests.simple", allow_json=True)]
+    [expose(template="turbogears.tests.simple", allow_json=True)]
     def flash_data_structure(self):
         flash(dict(uni=u"\xfcnicode", testing=[1, 2, 3]))
         return dict(title="Foobar", mybool=False, someval="niggles")
 
-    [expose(html="turbogears.tests.simple", allow_json=True)]
+    [expose(template="turbogears.tests.simple", allow_json=True)]
     def flash_redirect(self):
         flash(u"redirect \xfcnicode")
         redirect("/flash_redirected?tg_format=json")
 
-    [expose(html="turbogears.tests.simple", allow_json=True)]
+    [expose(template="turbogears.tests.simple", allow_json=True)]
     def flash_redirect_with_trouble_chars(self):
         flash(u"$foo, k\xe4se;\tbar!")
         redirect("/flash_redirected?tg_format=json")
 
-    [expose(html="turbogears.tests.simple", allow_json=True)]
+    [expose(template="turbogears.tests.simple", allow_json=True)]
     def flash_redirected(self):
         return dict(title="Foobar", mybool=False, someval="niggles")
 
@@ -244,8 +248,8 @@ class TestRoot(unittest.TestCase):
     def test_js_files(self):
         """Can access the JavaScript files"""
         testutil.create_request("/tg_js/MochiKit.js")
-        assert cherrypy.response.headers[
-            "Content-Type"] == "application/x-javascript"
+        assert cherrypy.response.headers["Content-Type"] in [
+            "application/javascript", "application/x-javascript"]
         assert cherrypy.response.status == "200 OK"
 
     def test_json_output(self):
@@ -277,10 +281,10 @@ class TestRoot(unittest.TestCase):
         """JSON output can be enabled via config."""
         config.update({'tg.allow_json':True})
         class JSONRoot(controllers.RootController):
-            [expose(html="turbogears.tests.simple")]
+            [expose(template="turbogears.tests.simple")]
             def allowjsonconfig(self):
                 return dict(title="Foobar", mybool=False, someval="foo",
-                     tg_html="turbogears.tests.simple")
+                     tg_template="turbogears.tests.simple")
         cherrypy.root = JSONRoot()
         testutil.create_request('/allowjsonconfig?tg_format=json')
         assert cherrypy.response.headers["Content-Type"] == "application/json"
@@ -290,10 +294,10 @@ class TestRoot(unittest.TestCase):
         """Make sure JSON can still be restricted with a global config on."""
         config.update({'tg.allow_json': True})
         class JSONRoot(controllers.RootController):
-            [expose(html="turbogears.tests.simple")]
+            [expose(template="turbogears.tests.simple")]
             def allowjsonconfig(self):
                 return dict(title="Foobar", mybool=False, someval="foo",
-                     tg_html="turbogears.tests.simple")
+                     tg_template="turbogears.tests.simple")
         cherrypy.root = JSONRoot()
         testutil.create_request('/allowjson?tg_format=json')
         assert cherrypy.response.headers["Content-Type"] == "text/html"
@@ -350,10 +354,14 @@ class TestRoot(unittest.TestCase):
         d = testutil.call(cherrypy.root.returnjson)
         assert d["title"] == "Foobar"
 
-    def test_templateOutput(self):
+    def test_template_output(self):
         testutil.create_request("/test")
         assert "Paging all niggles" in cherrypy.response.body[0]
 
+    def test_template_deprecated(self):
+        testutil.create_request("/test_deprecated")
+        assert "Paging all giggles" in cherrypy.response.body[0]
+
     def test_unicode(self):
         testutil.create_request("/unicode")
         firstline = cherrypy.response.body[0].split('\n')[0].decode('utf-8')
@@ -443,7 +451,7 @@ class TestRoot(unittest.TestCase):
         assert cherrypy.root.errors.has_key("firstname")
 
     def test_other_template(self):
-        """'tg_html' in a returned dict will use the template specified there"""
+        """'tg_template' in a returned dict will use the template specified there"""
         testutil.create_request("/useother")
         assert "This is the other template" in cherrypy.response.body[0]
 
@@ -600,6 +608,10 @@ class TestURLs(unittest.TestCase):
         cherrypy.root.subthing = SubApp()
         cherrypy.root.subthing.subsubthing = SubApp()
 
+    def tearDown(self):
+        config.update({"server.webpath": ""})
+        startup.startTurboGears()
+
     def test_basic_urls(self):
         testutil.create_request("/")
         assert "/foo" == url("/foo")
@@ -616,14 +628,19 @@ class TestURLs(unittest.TestCase):
 
     def test_approots(self):
         testutil.create_request("/subthing/")
+        assert cherrypy.response.status.startswith("200")
         assert url("foo") == "foo"
         assert url("/foo") == "/subthing/foo"
+        testutil.create_request("/nosubthing/")
+        assert cherrypy.response.status.startswith("404")
+        assert url("foo") == "foo"
+        assert url("/foo") == "/foo"
 
     def test_lower_approots(self):
         testutil.create_request("/subthing/subsubthing/")
         assert url("/foo") == "/subthing/subsubthing/foo"
 
-    def test_approots_With_path(self):
+    def test_approots_with_path(self):
         config.update({"server.webpath": "/coolsite/root"})
         startup.startTurboGears()
         testutil.create_request("/coolsite/root/subthing/")
@@ -643,6 +660,11 @@ class TestURLs(unittest.TestCase):
             assert False, "redirect exception should have been raised"
         except cherrypy.HTTPRedirect, e:
             assert "http://localhost/coolsite/root/subthing/foo" in e.urls
+        try:
+            redirect("foo")
+            assert False, "redirect exception should have been raised"
+        except cherrypy.HTTPRedirect, e:
+            assert "http://localhost/coolsite/root/subthing/foo" in e.urls
 
     def test_multi_values(self):
         testutil.create_request("/")
@@ -667,10 +689,22 @@ class TestURLs(unittest.TestCase):
         assert url('/', foo=['bar', u'\N{LATIN SMALL LETTER A WITH GRAVE}']
             ) == '/?foo=bar&foo=%C3%A0'
 
-    def tearDown(self):
-        config.update({"server.webpath": ""})
-        startup.startTurboGears()
+    def test_existing_query_string(self):
+        """url() can handle URL with existing query string"""
+        testutil.create_request("/")
+        test_url = url('/foo', {'first': 1})
+        assert url(test_url, {'second': 2}) == '/foo?first=1&second=2'
 
+    def test_url_kwargs_overwrite_tgparams(self):
+        """Keys in tgparams in call to url() overwrite kw args"""
+        params = {'spamm': 'eggs'}
+        assert 'spamm=ham' in url('/foo', params, spamm='ham')
+
+    def test_url_doesnt_change_tgparams(self):
+        """url() does not change the dict passed as second arg"""
+        params = {'spamm': 'eggs'}
+        assert 'foo' in url('/foo', params, spamm='ham')
+        assert params['spamm'] == 'eggs'
 
 def test_index_trailing_slash():
     """If there is no trailing slash on an index method call, redirect"""
@@ -694,14 +728,3 @@ def test_can_use_internally_defined_arguments():
     assert "format:foo" in output
     assert "template:bar" in output
     assert "fragment:boo" in output
-
-def test_url_kwargs_overwrite_tgparams():
-    """Check keys in tgparams in call to url overwrite kw args"""
-    params = {'spamm': 'eggs'}
-    assert 'spamm=ham' in url('/foo', params, spamm='ham')
-
-def test_url_doesnt_change_tgparams():
-    """Test that url() does not change the dict passed as second arg"""
-    params = {'spamm': 'eggs'}
-    assert 'foo' in url('/foo', params, spamm='ham')
-    assert params['spamm'] == 'eggs'
